#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef struct int_array {
  int *A;
  unsigned int length;
} int_array_t;

int_array_t init_int_array(const unsigned int length)
{
  int_array_t array;

  array.A = (int *)malloc(sizeof(int)*length);
  array.length = length; 

  return array;
}

void destroy_int_array(int_array_t *array)
{
  free(array->A);
  array->length = 0;
}

void printf_array(const int* A, const unsigned int length)
{
  if (length == 0) {
    printf("[]");

    return;
  }

  printf("[%d",A[0]);
  for (unsigned int i=1; i<length; i++) {
    printf(",%d",A[i]);
  }
  printf("]");
}

long int ipow(const long int base, const unsigned int exp)
{
   long int result=1;

   for (unsigned int i=0; i<exp; i++)
      result *= base;

   return result;
}

/* This is an auxiliary function: it takes as parameters 
   the arrays of subarrays, the number of already computed 
   subarrays, the array A, the length of A, the next index 
   to be considered in A, a temporary array tmp, meant to 
   store partially built subarrays of A, and the number of 
   "valid" element in tmp. 
   It returns the number of already computed subarrays of A. */
unsigned int get_sub_arrays_aux(int_array_t* subarrays, unsigned int next_sub_idx,
                                int* A, unsigned int A_length, unsigned int next_A_idx,
                                int* tmp, unsigned int next_tmp_idx
                               )
{
  /* if we filled the array of subarrays, then the computation must end */
  if (next_sub_idx == ipow(2,A_length)) {
     return next_sub_idx;
  }

  /* if all the elements of A has been considered, then tmp up to the index 
     next_tmp_idx-1 contains a subarray of A */ 
  if (next_A_idx == A_length) {
     /* so we can copy it into the array of subarrays */
     subarrays[next_sub_idx] = init_int_array(next_tmp_idx);
     
     memcpy(subarrays[next_sub_idx].A, tmp, sizeof(int)*next_tmp_idx);

     /* and increase the number of already built subarrays */
     next_sub_idx++;

     return next_sub_idx;
  }

  /* for each element (next_A_idx) of the array A either that element
     is not included into the subarray and we can skip it ... */
  next_sub_idx = get_sub_arrays_aux(subarrays, next_sub_idx, 
                                    A, A_length, next_A_idx+1,
                                    tmp, next_tmp_idx);

  /* ... or it is included and we must "enqueue" it into tmp */
  tmp[next_tmp_idx] = A[next_A_idx];
  next_sub_idx = get_sub_arrays_aux(subarrays, next_sub_idx, 
                                    A, A_length, next_A_idx+1,
                                    tmp, next_tmp_idx+1);

  return next_sub_idx;
}


/* This function prepares all the data structures required to the 
   real computation and invokes get_sub_arrays_aux */
int_array_t* get_sub_arrays(int* A, unsigned int length)
{
  /* the maximum length for a subarray of A is length, so, the tmp array 
     can have length length */
  int* tmp = (int *)malloc(sizeof(int)*length);

  /* The sub arrays of A are 2^A.length so we can allocate the space of 
     the array of subarrays */

  int_array_t* subarrays = (int_array_t*)calloc(sizeof(int_array_t), 
                                                ipow(2,length));
  
  get_sub_arrays_aux(subarrays, 0, A, length, 0, tmp, 0);

  free(tmp);

  return subarrays;
}



int main(int argc, char *argv[])
{
   int A[] = {3, -1, 6, 7, 12, 5, 1};
   unsigned int length = 7;

   printf("The subarrays of ");
   printf_array(A, length);
   printf(" are:\n");

   int_array_t* subarrays = get_sub_arrays(A, length);

   for (unsigned int i=0; i<ipow(2,length); i++) {
   	printf("%i) ", (i+1));
	printf_array(subarrays[i].A, subarrays[i].length);
   	printf("\n");
        destroy_int_array(&(subarrays[i]));
   }

   free(subarrays);

   return 0;
}

