#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void get_minimal_changes_aux(unsigned int n, 
                             unsigned int* C, unsigned int C_length, unsigned int C_idx, 
                             int *found, unsigned int* min_change, unsigned int* min_coins, 
                             unsigned int* tmp_change, unsigned int tmp_coins)
{
   /* if we already have a candidate change and the in-building change 
      has more coins than the candidate change, then back track*/
   if (*found && (*min_coins < tmp_coins)) {
     return;
   }

   /* if we discover an new change */
   if (n == 0) {

     /* if either this is the first change found or is minor than 
        the previous candidate, then... */  
     if ((!*found) || (*min_coins > tmp_coins)) {
        /* copy the new change on the candidate location*/
	      memcpy(min_change, tmp_change, sizeof(unsigned int)*C_length);

        /* update the minumum number of coins for the 
           candidate minimal change */
        *min_coins = tmp_coins;

        /* set the found flag to true */
        *found=1;
     }

     return;
   }

   /* if all the coins have been used */
   if (C_length == C_idx) {
     
     /* back track */
     return;
   }

   /* try to avoid further coins of the value C[C_idx] */
   get_minimal_changes_aux(n, C, C_length, C_idx+1, 
                           found, min_change, min_coins, 
                           tmp_change, tmp_coins);

   /* if C[C_idx] >= n, then also try to use one further 
      coin of this kind */
   if (n>=C[C_idx]) {

     /* increase by one the number of coins of this kind 
        included in the in-building change*/
     tmp_change[C_idx]++;
     
     /* try to build a valid change for n-C[C_idx] */
     get_minimal_changes_aux(n-C[C_idx], C, C_length, C_idx, 
                             found, min_change, min_coins, 
                             tmp_change, tmp_coins+1);

     /* back track and reset the in-building change as 
        it wasn't never changed */
     tmp_change[C_idx]--;
   }
}


unsigned int* get_minimal_changes(unsigned int n, unsigned int* C, unsigned int C_length)
{
   unsigned int* min_change=(unsigned int*)calloc(sizeof(unsigned int),C_length);
   unsigned int* tmp_change=(unsigned int*)calloc(sizeof(unsigned int),C_length);
   
   unsigned int min_coins;
   int found=0;

   get_minimal_changes_aux(n, C, C_length, 0, 
                           &found, min_change, &min_coins, 
                           tmp_change, 0);

   free(tmp_change);

   if (found) {
     return min_change;
   }

   return NULL;
}


void printf_change(unsigned int* C, unsigned int C_length, unsigned int *change)
{
  if (change==NULL) {

    printf("No change for the currency has been found\n");

    return;
  }
 
  char sep='{';
  for (unsigned int i=0; i<C_length; i++) {
    for (unsigned int j=0; j<change[i]; j++) {
      printf("%c%u", sep, C[i]);
      sep=',';
    }
  }

  if (sep=='{') {
    printf("{}");
  } else {
    printf("}");
  }
}

int main(int argc, char *argv[])
{
   unsigned int C[]={4, 3, 8, 11, 5};
   unsigned int C_length = 5;
   unsigned int n=17;

   unsigned int *min_change=get_minimal_changes(n, C, C_length);

   printf_change(C, C_length, min_change);
   
   printf("\n");

   free(min_change);

   return 0;
}

